Type Template
	Field sub_template.Template[7]                                  ;Sub templates
	Field emitter_blend                                             ;blendmode of emitter entity
	Field interval, particles_per_interval                          ;particle interval
	Field max_particles                                             ;max particles
	Field emitter_max_time                                          ;Emitter life time
	Field min_time, max_time                                        ;Particle life time
	Field tex, animtex, texframe#, maxtexframes, texspeed#          ;Texture
	Field min_ox#, max_ox#, min_oy#, max_oy#, min_oz#, max_oz#      ;Offset
	Field min_xv#, max_xv#, min_yv#, max_yv#, min_zv#, max_zv#      ;Velocity
	Field rot_vel1#, rot_vel2#, align_to_fall, align_to_fall_offset ;Rotation
	Field gravity#                                                  ;Gravity
	Field alpha#, alpha_vel                                         ;Alpha
	Field sx#, sy#, size_multiplicator1#, size_multiplicator2#      ;Size
	Field size_add#, size_mult#                                     ;Size velocity
	Field r1, g1, b1, r2, g2, b2                                    ;Colors
	Field brightness                                                ;Brightness
	Field floor_y#, floor_bounce#                                   ;Floor
	Field pitch_fix, yaw_fix                                        ;Fix angles
End Type
Type Emitter
	Field fixed
	Field cnt_loop, age, max_time
	Field tmp.Template
	Field owner, ent, surf
	Field del, frozen
End Type
Type Particle
	Field emitter.Emitter
	Field age, max_time  ;Life time
	Field x#, y#, z#     ;Position
	Field xv#, yv#, zv#  ;Velocity
	Field rot#, rot_vel# ;Rotation
	Field sx#, sy#       ;Size
End Type
Global ParticleCam, ParticlePiv

Function InitParticles(cam)
ParticleCam = cam
ParticlePiv = CreatePivot()
SeedRnd MilliSecs()
End Function

Function FreeParticles()
For tmp.Template = Each Template
	FreeTemplate(Handle(tmp))
Next
For ee.Emitter = Each Emitter
	FreeEmitter(ee\ent)
Next
Delete Each Template
Delete Each Emitter
Delete Each Particle
If ParticlePiv Then FreeEntity ParticlePiv
End Function

Function CreateTemplate()
tmp.Template = New Template
template = Handle(tmp)
SetTemplateEmitterBlend(template, 3)
SetTemplateInterval(template, 1)
SetTemplateParticlesPerInterval(template, 1)
SetTemplateMaxParticles(template, -1)
SetTemplateEmitterLifeTime(template, 100)
SetTemplateParticleLifeTime(template, 0, 20)
SetTemplateAlpha(template, 1)
SetTemplateSize(template, 1, 1)
SetTemplateSizeVel(template, 0, 1)
SetTemplateColors(template, $FFFFFF, $FFFFFF)
SetTemplateBrightness(template, 1)
SetTemplateFloor(template, -1000000)
SetTemplateFixAngles(template, -1, -1)
Return Handle(tmp)
End Function

Function FreeTemplate(template)
tmp.Template = Object.Template(template)
If tmp\tex Then FreeTexture tmp\tex
For i = 0 To 7
	If tmp\sub_template[i] <> Null Then FreeTemplate(Handle(tmp\sub_template[i]))
Next
Delete tmp
End Function

Function SetTemplateEmitterBlend(template, emitter_blend)
tmp.Template = Object.Template(template)
tmp\emitter_blend = emitter_blend
End Function

Function SetTemplateInterval(template, interval)
tmp.Template = Object.Template(template)
tmp\interval = interval
End Function

Function SetTemplateParticlesPerInterval(template, particles_per_interval)
tmp.Template = Object.Template(template)
tmp\particles_per_interval = particles_per_interval
End Function

Function SetTemplateMaxParticles(template, max_particles)
tmp.Template = Object.Template(template)
tmp\max_particles = max_particles
End Function

Function SetTemplateParticleLifeTime(template, min_time, max_time)
tmp.Template = Object.Template(template)
tmp\min_time = min_time
tmp\max_time = max_time
End Function

Function SetTemplateEmitterLifeTime(template, emitter_max_time)
tmp.Template = Object.Template(template)
tmp\emitter_max_time = emitter_max_time
End Function

Function SetTemplateTexture(template, path$, mode = 0, blend = 1)
tmp.Template = Object.Template(template)
tmp\tex = LoadTexture(path$, mode)
TextureBlend tmp\tex, blend
End Function

Function SetTemplateAnimTexture(template, path$, mode, blend, w, h, maxframes, speed# = 1)
tmp.Template = Object.Template(template)
tmp\animtex = True
tmp\maxtexframes = maxframes
tmp\texspeed# = speed#
tmp\tex = LoadAnimTexture(path$, mode, w, h, 0, tmp\maxtexframes)
TextureBlend tmp\tex, blend
End Function

Function SetTemplateOffset(template, min_ox#, max_ox#, min_oy#, max_oy#, min_oz#, max_oz#)
tmp.Template = Object.Template(template)
tmp\min_ox# = min_ox#
tmp\max_ox# = max_ox#
tmp\min_oy# = min_oy#
tmp\max_oy# = max_oy#
tmp\min_oz# = min_oz#
tmp\max_oz# = max_oz#
End Function

Function SetTemplateVelocity(template, min_xv#, max_xv#, min_yv#, max_yv#, min_zv#, max_zv#)
tmp.Template = Object.Template(template)
tmp\min_xv# = min_xv#
tmp\max_xv# = max_xv#
tmp\min_yv# = min_yv#
tmp\max_yv# = max_yv#
tmp\min_zv# = min_zv#
tmp\max_zv# = max_zv#
End Function

Function SetTemplateRotation(template, rot_vel1#, rot_vel2#)
tmp.Template = Object.Template(template)
tmp\rot_vel1# = rot_vel1#
tmp\rot_vel2# = rot_vel2#
End Function

Function SetTemplateAlignToFall(template, align_to_fall, align_to_fall_offset = 0)
tmp.Template = Object.Template(template)
tmp\align_to_fall = align_to_fall
tmp\align_to_fall_offset = align_to_fall_offset
End Function

Function SetTemplateGravity(template, gravity#)
tmp.Template = Object.Template(template)
tmp\gravity# = gravity#
End Function

Function SetTemplateSize(template, sx#, sy#, size_multiplicator1# = 1, size_multiplicator2# = 1)
tmp.Template = Object.Template(template)
tmp\sx# = sx#
tmp\sy# = sy#
tmp\size_multiplicator1# = size_multiplicator1#
tmp\size_multiplicator2# = size_multiplicator2#
End Function

Function SetTemplateSizeVel(template, size_add#, size_mult#)
tmp.Template = Object.Template(template)
tmp\size_add# = size_add#
tmp\size_mult# = size_mult#
End Function

Function SetTemplateAlpha(template, alpha#)
tmp.Template = Object.Template(template)
tmp\alpha# = alpha#
End Function

Function SetTemplateAlphaVel(template, alpha_vel)
tmp.Template = Object.Template(template)
tmp\alpha_vel = alpha_vel
End Function

Function SetTemplateColors(template, col1, col2)
tmp.Template = Object.Template(template)
tmp\r1 = (col1 And $FF0000) / $10000
tmp\g1 = (col1 And $FF00) / $100
tmp\b1 = col1 And $FF
tmp\r2 = (col2 And $FF0000) / $10000
tmp\g2 = (col2 And $FF00) / $100
tmp\b2 = col2 And $FF
End Function

Function SetTemplateBrightness(template, brightness)
tmp.Template = Object.Template(template)
tmp\brightness = brightness
End Function

Function SetTemplateFloor(template, floor_y#, floor_bounce# = .5)
tmp.Template = Object.Template(template)
tmp\floor_y# = floor_y#
tmp\floor_bounce# = floor_bounce#
End Function

Function SetTemplateFixAngles(template, pitch_fix, yaw_fix)
tmp.Template = Object.Template(template)
tmp\pitch_fix = pitch_fix
tmp\yaw_fix = yaw_fix
End Function

Function SetTemplateSubTemplate(template, sub_template, for_each_particle = False)
tmp.Template = Object.Template(template)
For i = 0 To 7
	If tmp\sub_template[i] = Null Then
		tmp\sub_template[i] = Object.Template(sub_template)
		Exit
	EndIf
Next
End Function

Function SetEmitter(owner, template, fixed = False)
	ee.Emitter = New Emitter
If fixed Then
	ee\owner = CreatePivot()
	PositionEntity ee\owner, EntityX(owner), EntityY(owner), EntityZ(owner)
	ee\fixed = True
Else
	ee\owner = owner
EndIf
ee\ent = CreateMesh()
ee\surf = CreateSurface(ee\ent)
ee\tmp = Object.Template(template)
ee\max_time = ee\tmp\emitter_max_time
EntityBlend ee\ent, ee\tmp\emitter_blend
EntityFX ee\ent, 34
If ee\tmp\tex Then EntityTexture ee\ent, ee\tmp\tex
For i = 0 To 7
	If ee\tmp\sub_template[i] <> Null Then
		If ee\tmp\sub_template[i]\tex Then SetEmitter(owner, Handle(ee\tmp\sub_template[i]), fixed)
	EndIf
Next
Return ee\ent
End Function

Function FreeEmitter(ent, delete_particles = True)
	For ee.Emitter = Each Emitter
		If ee\owner = ent Then
			If delete_particles Then
				For pp.Particle = Each Particle
					If pp\emitter = ee Then Delete pp
				Next
				FreeEntity ee\ent
				If ee\fixed And ee\owner Then FreeEntity ee\owner
				Delete ee
			Else
				ee\del = True
			EndIf
		EndIf
	Next
End Function

Function FreezeEmitter(ent)
	For ee.Emitter = Each Emitter
		If ee\owner = ent Then ee\frozen = True
Next
End Function

Function UnfreezeEmitter(ent)
	For ee.Emitter = Each Emitter
		If ee\owner = ent Then ee\frozen = False
Next
End Function

Function UpdateParticles()
	For ee.Emitter = Each Emitter
		If ee\tmp\max_particles > -1 Then
			cnt_particles = 0
			For pp.Particle = Each Particle
				If pp\emitter = ee Then cnt_particles = cnt_particles + 1
			Next
		EndIf
		ClearSurface ee\surf
		If ee\max_time > -1 Then
			If ee\age > ee\max_time Then ee\del = True Else ee\age = ee\age + 1
		EndIf
		If ee\frozen = False Then
			ee\cnt_loop = (ee\cnt_loop + 1) Mod ee\tmp\interval
			If ee\cnt_loop = 0 And ee\del = False Then
				For i = 1 To ee\tmp\particles_per_interval
					If (ee\tmp\max_particles > -1 And cnt_particles < ee\tmp\max_particles) Or ee\tmp\max_particles = -1 Then
						pp.Particle = New Particle
						pp\emitter = ee
						pp\max_time = Rand(ee\tmp\min_time, ee\tmp\max_time)
						pp\x# = EntityX(ee\owner, True) + Rnd(ee\tmp\min_ox#, ee\tmp\max_ox#)
						pp\y# = EntityY(ee\owner, True) + Rnd(ee\tmp\min_oy#, ee\tmp\max_oy#)
						pp\z# = EntityZ(ee\owner, True) + Rnd(ee\tmp\min_oz#, ee\tmp\max_oz#)
						pp\xv# = Rnd(ee\tmp\min_xv#, ee\tmp\max_xv#)
						pp\yv# = Rnd(ee\tmp\min_yv#, ee\tmp\max_yv#)
						pp\zv# = Rnd(ee\tmp\min_zv#, ee\tmp\max_zv#)
						pp\rot_vel# = Rnd(ee\tmp\rot_vel1#, ee\tmp\rot_vel2#)
						sm# = Rnd(ee\tmp\size_multiplicator1#, ee\tmp\size_multiplicator2#)
						pp\sx# = pp\emitter\tmp\sx# * sm#
						pp\sy# = pp\emitter\tmp\sy# * sm#
					EndIf
				Next
			EndIf
		EndIf
		If ee\tmp\animtex Then
			ee\tmp\texframe# = ee\tmp\texframe# + ee\tmp\texspeed#
			If ee\tmp\texframe# > ee\tmp\maxtexframes - 1 Then ee\tmp\texframe# = 0
			EntityTexture ee\ent, ee\tmp\tex, ee\tmp\texframe#
		EndIf
		frame = frame + texspeed#
		If ee\del Then
			del = True
			For pp.Particle = Each Particle
				If pp\emitter = ee Then del = False
			Next
			If del Then
				FreeEntity ee\ent
				If ee\fixed And ee\owner Then FreeEntity ee\owner
				Delete ee
			EndIf
		EndIf
	Next
	PositionEntity ParticlePiv, EntityX(ParticleCam, True), EntityY(ParticleCam, True), EntityZ(ParticleCam, True)
	cam_pitch# = EntityPitch(ParticleCam, True)
	cam_yaw# = EntityYaw(ParticleCam, True)
	cam_roll# = EntityRoll(ParticleCam, True)
	For pp.Particle = Each Particle
		If pp\age > pp\max_time Then
			Delete pp
		Else
			If pp\emitter\frozen = False Then
				pp\age = pp\age + 1
				If pp\emitter\tmp\align_to_fall Then pp\rot# = pp\emitter\tmp\align_to_fall_offset - ATan2(pp\xv#, pp\yv#) Else pp\rot# = pp\rot# + pp\rot_vel#
				pp\yv# = pp\yv# - pp\emitter\tmp\gravity#
				pp\x# = pp\x# + pp\xv#
				pp\y# = pp\y# + pp\yv#
				pp\z# = pp\z# + pp\zv#
				If pp\y# < pp\emitter\tmp\floor_y# Then pp\yv# = pp\yv# * -pp\emitter\tmp\floor_bounce#
				pp\sx# = (pp\sx# + pp\emitter\tmp\size_add#) * pp\emitter\tmp\size_mult#
				pp\sy# = (pp\sy# + pp\emitter\tmp\size_add#) * pp\emitter\tmp\size_mult#
			EndIf
			RotateEntity ParticlePiv, cam_pitch#, cam_yaw#, cam_roll# + pp\rot# + pp\emitter\tmp\align_to_fall_offset
			If pp\emitter\tmp\pitch_fix > -1 Then RotateEntity ParticlePiv, pp\emitter\tmp\pitch_fix, EntityYaw(ParticlePiv), EntityRoll(ParticlePiv)
			If pp\emitter\tmp\yaw_fix > -1 Then RotateEntity ParticlePiv, EntityPitch(ParticlePiv), pp\emitter\tmp\yaw_fix, EntityRoll(ParticlePiv)
			x# = EntityX(pp\emitter\ent) + pp\x#
			y# = EntityY(pp\emitter\ent) + pp\y#
			z# = EntityZ(pp\emitter\ent) + pp\z#
			sx# = pp\sx#
			sy# = pp\sy#
			TFormVector sx#, -sy#, 0, ParticlePiv, 0
			v1x# = TFormedX() + x#
			v1y# = TFormedY() + y#
			v1z# = TFormedZ() + z#
			TFormVector -sx#, -sy#, 0, ParticlePiv, 0
			v2x# = TFormedX() + x#
			v2y# = TFormedY() + y#
			v2z# = TFormedZ() + z#
			TFormVector sx#, sy#, 0, ParticlePiv, 0
			v3x# = TFormedX() + x#
			v3y# = TFormedY() + y#
			v3z# = TFormedZ() + z#
			TFormVector -sx#, sy#, 0, ParticlePiv, 0
			v4x# = TFormedX() + x#
			v4y# = TFormedY() + y#
			v4z# = TFormedZ() + z#
			v1 = AddVertex(pp\emitter\surf, v1x#, v1y#, v1z#, 0, 0)
			v2 = AddVertex(pp\emitter\surf, v2x#, v2y#, v2z#, 1, 0)
			v3 = AddVertex(pp\emitter\surf, v3x#, v3y#, v3z#, 0, 1)
			v4 = AddVertex(pp\emitter\surf, v4x#, v4y#, v4z#, 1, 1)
			rr = pp\emitter\tmp\r1 + (pp\emitter\tmp\r2 - pp\emitter\tmp\r1) * Float(pp\age) / Float(pp\max_time)
			gg = pp\emitter\tmp\g1 + (pp\emitter\tmp\g2 - pp\emitter\tmp\g1) * Float(pp\age) / Float(pp\max_time)
			bb = pp\emitter\tmp\b1 + (pp\emitter\tmp\b2 - pp\emitter\tmp\b1) * Float(pp\age) / Float(pp\max_time)
			If pp\emitter\tmp\alpha_vel Then a# = (1 - Float(pp\age) / Float(pp\max_time)) * pp\emitter\tmp\alpha# Else a# = pp\emitter\tmp\alpha#
			VertexColor pp\emitter\surf, v1, rr, gg, bb, a#
			VertexColor pp\emitter\surf, v2, rr, gg, bb, a#
			VertexColor pp\emitter\surf, v3, rr, gg, bb, a#
			VertexColor pp\emitter\surf, v4, rr, gg, bb, a#
			For i = 1 To pp\emitter\tmp\brightness
				AddTriangle pp\emitter\surf, v1, v2, v3
				AddTriangle pp\emitter\surf, v3, v2, v4
			Next
		EndIf
	Next
End Function
;~IDEal Editor Parameters:
;~F#0#14#1B#25#2B#38#4B#54#59#5E#63#68#6E#73#79#82#8C#96#9C#A2
;~F#A7#AF#B5#BA#BF#C9#CE#D4#DA#E4#10D#113#119
;~C#Blitz3D